@@ -2,7 +2,7 @@
  * ex: set ts=8: */
 /* Evolution calendar - caldav backend
  *
- * Copyright (C) 2005 Novell, Inc.
+ * Copyright (C) 1999-2008 Novell, Inc. (www.novell.com)
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of version 2 of the GNU General Public
@@ -1611,10 +1611,9 @@ caldav_remove (ECalBackendSync *backend,
 
 	status = check_state (cbdav, &online);
 
-	if (status != GNOME_Evolution_Calendar_Success) {
-		g_mutex_unlock (priv->lock);
-		return status;
-	}
+	/* lie here a bit, but otherwise the calendar will not be removed, even it should */
+	if (status != GNOME_Evolution_Calendar_Success)
+		g_print (G_STRLOC ": %s", e_cal_backend_status_to_string (status));
 
 	e_file_cache_remove (E_FILE_CACHE (priv->cache));
 	priv->cache  = NULL;
@@ -1834,11 +1833,22 @@ caldav_remove_object (ECalBackendSync  *backend,
 	}
 
 	cache_comp = e_cal_backend_cache_get_component (priv->cache, uid, rid);
+
+	if (cache_comp == NULL && rid && *rid) {
+		/* we do not have this instance in cache directly, thus try to get master object */
+		cache_comp = e_cal_backend_cache_get_component (priv->cache, uid, "");
+	}
+
 	if (cache_comp == NULL) {
 		g_mutex_unlock (priv->lock);
 		return GNOME_Evolution_Calendar_ObjectNotFound;
 	}
 
+	*old_object = e_cal_component_get_as_string (cache_comp);
+
+	if (mod == CALOBJ_MOD_THIS)
+		e_cal_util_remove_instances (e_cal_component_get_icalcomponent (cache_comp), icaltime_from_string (rid), mod);
+
 	if (online) {
 		CalDAVObject caldav_object;
 
@@ -1846,14 +1856,20 @@ caldav_remove_object (ECalBackendSync  *backend,
 		caldav_object.etag  = e_cal_component_get_etag (cache_comp);
 		caldav_object.cdata = NULL;
 
-		status = caldav_server_delete_object (cbdav, &caldav_object);
+		if (mod == CALOBJ_MOD_THIS) {
+			caldav_object.cdata = pack_cobj (cbdav, cache_comp);
+
+			status = caldav_server_put_object (cbdav, &caldav_object);
+		} else
+			status = caldav_server_delete_object (cbdav, &caldav_object);
 
 		caldav_object_free (&caldav_object, FALSE);
-
 	} else {
 		/* mark component as out of synch */
-		e_cal_component_set_synch_state (cache_comp,
-				E_CAL_COMPONENT_LOCALLY_DELETED);
+		if (mod == CALOBJ_MOD_THIS)
+			e_cal_component_set_synch_state (cache_comp, E_CAL_COMPONENT_LOCALLY_MODIFIED);
+		else
+			e_cal_component_set_synch_state (cache_comp, E_CAL_COMPONENT_LOCALLY_DELETED);
 	}
 
 	if (status != GNOME_Evolution_Calendar_Success) {
@@ -1861,13 +1877,13 @@ caldav_remove_object (ECalBackendSync  *backend,
 		return status;
 	}
 
-	*old_object = e_cal_component_get_as_string (cache_comp);
-
 	/* We should prolly check for cache errors
 	 * but when that happens we are kinda hosed anyway */
-	e_cal_backend_cache_remove_component (priv->cache, uid, rid);
-
-	/* FIXME: set new_object when removing instances of a recurring appointment */
+	if (mod == CALOBJ_MOD_THIS) {
+		e_cal_backend_cache_put_component (priv->cache, cache_comp);
+		*object = e_cal_component_get_as_string (cache_comp);
+	} else
+		e_cal_backend_cache_remove_component (priv->cache, uid, rid);
 
 	g_mutex_unlock (priv->lock);
 
@@ -1937,7 +1953,7 @@ process_object (ECalBackendCalDAV   *cbdav,
 	struct icaltimetype       now;
 	ECalComponentId          *id;
 	const char               *uid;
-	const char               *rid;
+	char                     *rid;
 	char                     *ostr;
 	char                     *oostr;
 
@@ -2083,6 +2099,7 @@ process_object (ECalBackendCalDAV   *cbdav,
 	g_object_unref (ecomp);
 	g_free (ostr);
 	g_free (oostr);
+	g_free (rid);
 
 	if (ccomp) {
 		g_object_unref (ccomp);
