@@ -1,8 +1,7 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /* Evolution calendar - generic backend class
  *
- * Copyright (C) 2000 Ximian, Inc.
- * Copyright (C) 2000 Ximian, Inc.
+ * Copyright (C) 1999-2008 Novell, Inc. (www.novell.com)
  *
  * Authors: Federico Mena-Quintero <federico@ximian.com>
  *          JP Rosevear <jpr@ximian.com>
@@ -51,6 +50,11 @@ struct _ECalBackendPrivate {
 
 	/* ECalBackend to pass notifications on to */
 	ECalBackend *notification_proxy;
+
+	/* used when notifying clients about progress of some operation,
+	 * we do not send multiple notifications with the same percent
+	 * value */
+	int last_percent_notified;
 };
 
 /* Property IDs */
@@ -277,6 +281,7 @@ e_cal_backend_init (ECalBackend *backend)
 
 	priv->clients = NULL;
 	priv->clients_mutex = g_mutex_new ();
+	priv->last_percent_notified = 0;
 
 	/* FIXME bonobo_object_ref/unref? */
 	priv->queries = e_list_new ((EListCopyFunc) bonobo_object_ref, (EListFreeFunc) bonobo_object_unref, NULL);
@@ -1200,6 +1205,23 @@ match_query_and_notify (EDataCalView *query, const char *old_object, const char
 	}
 }
 
+/**
+ * e_cal_backend_notify_view_progress_start
+ * @backend: A calendar backend.
+ *
+ * This methods has to be used before e_cal_backend_notify_view_progress.
+ * Sets last notified percent value to 0.
+ **/
+void
+e_cal_backend_notify_view_progress_start (ECalBackend *backend)
+{
+	ECalBackendPrivate *priv;
+
+	priv = backend->priv;
+
+	priv->last_percent_notified = 0;
+}
+
 /**
  * e_cal_backend_notify_view_progress:
  * @backend: A calendar backend.
@@ -1218,6 +1240,11 @@ e_cal_backend_notify_view_progress (ECalBackend *backend, const char *message, i
 
 	priv = backend->priv;
 
+	if (percent <= priv->last_percent_notified)
+		return;
+
+	priv->last_percent_notified = percent;
+
 	if (priv->notification_proxy) {
 		e_cal_backend_notify_view_progress (priv->notification_proxy, message, percent);
 		return;
@@ -1376,6 +1403,25 @@ e_cal_backend_notify_object_removed (ECalBackend *backend, const ECalComponentId
 	g_object_unref (iter);
 }
 
+
+void 
+e_cal_backend_notify_objects_added (ECalBackend *backend, EDataCalView *query, const GList *objects)
+{
+	e_data_cal_view_notify_objects_added (query, objects);	
+}
+
+void 
+e_cal_backend_notify_objects_removed (ECalBackend *backend, EDataCalView *query, const GList *ids)
+{
+	e_data_cal_view_notify_objects_removed (query, ids);	
+}
+
+void 
+e_cal_backend_notify_objects_modified (ECalBackend *backend, EDataCalView *query, const GList *objects)
+{
+	e_data_cal_view_notify_objects_modified (query, objects);	
+}
+
 /**
  * e_cal_backend_notify_mode:
  * @backend: A calendar backend.
