@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8; fill-column: 160 -*- */
 /*
- *  Copyright (C) 2000 Ximian Inc.
+ *  Copyright (C) 1999-2008 Novell, Inc. (www.novell.com)
  *
  *  Authors: Michael Zucchi <notzed@ximian.com>
  *
@@ -43,6 +43,7 @@
 #include "camel-nntp-store.h"
 #include "camel-nntp-stream.h"
 #include "camel-nntp-summary.h"
+#include "camel-string-utils.h"
 
 #define w(x)
 #define io(x)
@@ -51,6 +52,9 @@
 
 #define CAMEL_NNTP_SUMMARY_VERSION (1)
 
+#define EXTRACT_FIRST_DIGIT(val) val=strtoul (part, &part, 10);
+#define EXTRACT_DIGIT(val) part++; val=strtoul (part, &part, 10);
+
 struct _CamelNNTPSummaryPrivate {
 	char *uid;
 
@@ -63,6 +67,8 @@ struct _CamelNNTPSummaryPrivate {
 static CamelMessageInfo * message_info_new_from_header (CamelFolderSummary *, struct _camel_header_raw *);
 static int summary_header_load(CamelFolderSummary *, FILE *);
 static int summary_header_save(CamelFolderSummary *, FILE *);
+static int summary_header_from_db (CamelFolderSummary *s, CamelFIRecord *mir);
+static CamelFIRecord * summary_header_to_db (CamelFolderSummary *s, CamelException *ex);
 
 static void camel_nntp_summary_class_init (CamelNNTPSummaryClass *klass);
 static void camel_nntp_summary_init       (CamelNNTPSummary *obj);
@@ -73,7 +79,7 @@ CamelType
 camel_nntp_summary_get_type(void)
 {
 	static CamelType type = CAMEL_INVALID_TYPE;
-
+	
 	if (type == CAMEL_INVALID_TYPE) {
 		type = camel_type_register(camel_folder_summary_get_type(), "CamelNNTPSummary",
 					   sizeof (CamelNNTPSummary),
@@ -83,7 +89,7 @@ camel_nntp_summary_get_type(void)
 					   (CamelObjectInitFunc) camel_nntp_summary_init,
 					   (CamelObjectFinalizeFunc) camel_nntp_summary_finalise);
 	}
-
+	
 	return type;
 }
 
@@ -91,12 +97,14 @@ static void
 camel_nntp_summary_class_init(CamelNNTPSummaryClass *klass)
 {
 	CamelFolderSummaryClass *sklass = (CamelFolderSummaryClass *) klass;
-
+	
 	camel_nntp_summary_parent = CAMEL_FOLDER_SUMMARY_CLASS(camel_type_get_global_classfuncs(camel_folder_summary_get_type()));
 
 	sklass->message_info_new_from_header  = message_info_new_from_header;
 	sklass->summary_header_load = summary_header_load;
 	sklass->summary_header_save = summary_header_save;
+	sklass->summary_header_from_db = summary_header_from_db;
+	sklass->summary_header_to_db = summary_header_to_db;
 }
 
 static void
@@ -132,7 +140,7 @@ camel_nntp_summary_new(struct _CamelFolder *folder, const char *path)
 
 	camel_folder_summary_set_filename((CamelFolderSummary *)cns, path);
 	camel_folder_summary_set_build_content((CamelFolderSummary *)cns, FALSE);
-
+	
 	return cns;
 }
 
@@ -151,14 +159,42 @@ message_info_new_from_header(CamelFolderSummary *s, struct _camel_header_raw *h)
 
 	mi = (CamelMessageInfoBase *)((CamelFolderSummaryClass *)camel_nntp_summary_parent)->message_info_new_from_header(s, h);
 	if (mi) {
-		g_free(mi->uid);
-		mi->uid = cns->priv->uid;
+		camel_pstring_free(mi->uid);
+		mi->uid = camel_pstring_strdup(cns->priv->uid);
+		g_free(cns->priv->uid);
 		cns->priv->uid = NULL;
 	}
-
+	
 	return (CamelMessageInfo *)mi;
 }
 
+static int
+summary_header_from_db (CamelFolderSummary *s, CamelFIRecord *mir)
+{
+	CamelNNTPSummary *cns = CAMEL_NNTP_SUMMARY(s);
+	char *part;
+
+	
+	if (camel_nntp_summary_parent->summary_header_from_db (s, mir) == -1)
+		return -1;
+
+	part = mir->bdata;
+
+	if (part) {
+		EXTRACT_FIRST_DIGIT (cns->version)
+	}
+	
+	if (part) {
+		EXTRACT_DIGIT (cns->high)
+	}
+
+	if (part) {
+		EXTRACT_DIGIT (cns->low)
+	}
+
+	return 0;
+}
+
 static int
 summary_header_load(CamelFolderSummary *s, FILE *in)
 {
@@ -189,6 +225,21 @@ summary_header_load(CamelFolderSummary *s, FILE *in)
 	return 0;
 }
 
+
+static CamelFIRecord *
+summary_header_to_db (CamelFolderSummary *s, CamelException *ex)
+{
+	CamelNNTPSummary *cns = CAMEL_NNTP_SUMMARY(s);
+	struct _CamelFIRecord *fir;
+	
+	fir = camel_nntp_summary_parent->summary_header_to_db (s, ex);
+	if (!fir)
+		return NULL;
+	fir->bdata = g_strdup_printf ("%d %d %d", CAMEL_NNTP_SUMMARY_VERSION, cns->high, cns->low);
+
+	return fir;
+}
+
 static int
 summary_header_save(CamelFolderSummary *s, FILE *out)
 {
@@ -336,7 +387,7 @@ add_range_head(CamelNNTPSummary *cns, CamelNNTPStore *store, unsigned int high,
 		n = strtoul(line, &line, 10);
 		if (n != i)
 			g_warning("retrieved message '%u' when i expected '%u'?\n", n, i);
-
+		
 		/* FIXME: use camel-mime-utils.c function for parsing msgid? */
 		if ((msgid = strchr(line, '<')) && (line = strchr(msgid+1, '>'))){
 			line[1] = 0;
@@ -427,28 +478,24 @@ camel_nntp_summary_check(CamelNNTPSummary *cns, CamelNNTPStore *store, char *lin
 	if (cns->low != f) {
 		count = camel_folder_summary_count(s);
 		for (i = 0; i < count; i++) {
-			CamelMessageInfo *mi = camel_folder_summary_index(s, i);
+			const char *uid;
+			const char *msgid;
 
-			if (mi) {
-				const char *uid = camel_message_info_uid(mi);
-				const char *msgid;
+			uid  = camel_folder_summary_uid_from_index(s, i);
+			n = strtoul(uid, NULL, 10);
 
-				n = strtoul(uid, NULL, 10);
-				if (n < f || n > l) {
-					dd(printf("nntp_summary: %u is lower/higher than lowest/highest article, removed\n", n));
-					/* Since we use a global cache this could prematurely remove
-					   a cached message that might be in another folder - not that important as
-					   it is a true cache */
-					msgid = strchr(uid, ',');
-					if (msgid)
-						camel_data_cache_remove(store->cache, "cache", msgid+1, NULL);
-					camel_folder_change_info_remove_uid(changes, uid);
-					camel_folder_summary_remove(s, mi);
-					count--;
-					i--;
-				}
-
-				camel_message_info_free(mi);
+			if (n < f || n > l) {
+				dd(printf("nntp_summary: %u is lower/higher than lowest/highest article, removed\n", n));
+				/* Since we use a global cache this could prematurely remove
+				   a cached message that might be in another folder - not that important as
+				   it is a true cache */
+				msgid = strchr(uid, ',');
+				if (msgid)
+					camel_data_cache_remove(store->cache, "cache", msgid+1, NULL);
+				camel_folder_change_info_remove_uid(changes, uid);
+				camel_folder_summary_remove_uid (s, uid);
+				count--;
+				i--;
 			}
 		}
 		cns->low = f;
@@ -467,7 +514,7 @@ camel_nntp_summary_check(CamelNNTPSummary *cns, CamelNNTPStore *store, char *lin
 
 	/* TODO: not from here */
 	camel_folder_summary_touch(s);
-	camel_folder_summary_save(s);
+	camel_folder_summary_save_to_db (s, ex);
 update:
 	/* update store summary if we have it */
 	if (folder
@@ -484,7 +531,7 @@ update:
 				camel_message_info_free(mi);
 			}
 		}
-
+		
 		if (si->info.unread != unread
 		    || si->info.total != count
 		    || si->first != f
