@@ -2,7 +2,7 @@
 
 /* e-contact-store.c - Contacts store with GtkTreeModel interface.
  *
- * Copyright (C) 2004 Novell, Inc.
+ * Copyright (C) 1999-2008 Novell, Inc. (www.novell.com)
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of version 2 of the GNU Lesser General Public
@@ -15,7 +15,7 @@
  *
  * You should have received a copy of the GNU Lesser General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
  * Authors: Hans Petter Jansson <hpj@novell.com>
  */
@@ -25,9 +25,6 @@
 #endif
 
 #include <string.h>
-#include <gtk/gtktreemodel.h>
-#include <gtk/gtksignal.h>
-#include <gtk/gtktreednd.h>
 #include <glib/gi18n-lib.h>
 #include "e-contact-store.h"
 
@@ -43,7 +40,7 @@ static void         e_contact_store_init            (EContactStore      *contact
 static void         e_contact_store_class_init      (EContactStoreClass *class);
 static void         e_contact_store_tree_model_init (GtkTreeModelIface  *iface);
 static void         e_contact_store_finalize        (GObject            *object);
-static guint        e_contact_store_get_flags       (GtkTreeModel       *tree_model);
+static GtkTreeModelFlags e_contact_store_get_flags       (GtkTreeModel       *tree_model);
 static gint         e_contact_store_get_n_columns   (GtkTreeModel       *tree_model);
 static GType        e_contact_store_get_column_type (GtkTreeModel       *tree_model,
 						     gint                index);
@@ -73,8 +70,6 @@ static gboolean     e_contact_store_iter_parent     (GtkTreeModel       *tree_mo
 						     GtkTreeIter        *iter,
 						     GtkTreeIter        *child);
 
-static void stop_view (EContactStore *contact_store, EBookView *view);
-
 typedef struct
 {
 	EBook     *book;
@@ -87,6 +82,10 @@ typedef struct
 }
 ContactSource;
 
+static void free_contact_ptrarray (GPtrArray *contacts);
+static void clear_contact_source  (EContactStore *contact_store, ContactSource *source);
+static void stop_view             (EContactStore *contact_store, EBookView *view);
+
 /* ------------------ *
  * Class/object setup *
  * ------------------ */
@@ -168,7 +167,23 @@ e_contact_store_init (EContactStore *contact_store)
 static void
 e_contact_store_finalize (GObject *object)
 {
-	/* TODO: Free stuff */
+	EContactStore *contact_store = E_CONTACT_STORE (object);
+	gint           i;
+
+	/* Free sources and cached contacts */
+
+	for (i = 0; i < contact_store->contact_sources->len; i++) {
+		ContactSource *source = &g_array_index (contact_store->contact_sources, ContactSource, i);
+
+		clear_contact_source (contact_store, source);
+
+		free_contact_ptrarray (source->contacts);
+		g_object_unref (source->book);
+	}
+
+	g_array_free (contact_store->contact_sources, TRUE);
+	if (contact_store->query)
+		e_book_query_unref (contact_store->query);
 
 	if (G_OBJECT_CLASS (parent_class)->finalize)
 		(* G_OBJECT_CLASS (parent_class)->finalize) (object);
@@ -584,7 +599,7 @@ view_sequence_complete (EContactStore *contact_store, EBookViewStatus status, EB
 
 	/* However, if it was a pending view, calculate and emit the differences between that
 	 * and the current view, and move the pending view up to current.
-	 * 
+	 *
 	 * This is O(m * n), and can be sped up with a temporary hash table if needed. */
 
 	/* Deletions */
@@ -826,7 +841,7 @@ e_contact_store_get_contact (EContactStore *contact_store, GtkTreeIter *iter)
  * @iter: a destination #GtkTreeIter to set
  *
  * Sets @iter to point to the contact row matching @uid.
- * 
+ *
  * Return value: %TRUE if the contact was found, and @iter was set. %FALSE otherwise.
  **/
 gboolean
@@ -990,7 +1005,7 @@ e_contact_store_peek_query (EContactStore *contact_store)
  * GtkTreeModel API *
  * ---------------- */
 
-static guint
+static GtkTreeModelFlags
 e_contact_store_get_flags (GtkTreeModel *tree_model)
 {
 	g_return_val_if_fail (E_IS_CONTACT_STORE (tree_model), 0);
