@@ -2,7 +2,7 @@
 
 /* e-destination-store.c - EDestination store with GtkTreeModel interface.
  *
- * Copyright (C) 2004 Novell, Inc.
+ * Copyright (C) 1999-2008 Novell, Inc. (www.novell.com)
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of version 2 of the GNU Lesser General Public
@@ -25,9 +25,6 @@
 #endif
 
 #include <string.h>
-#include <gtk/gtktreemodel.h>
-#include <gtk/gtksignal.h>
-#include <gtk/gtktreednd.h>
 #include <glib/gi18n-lib.h>
 #include "e-destination-store.h"
 
@@ -354,7 +351,7 @@ e_destination_store_list_destinations (EDestinationStore *destination_store)
 		EDestination *destination = g_ptr_array_index (destination_store->destinations, i);
 		destination_list = g_list_prepend (destination_list, destination);
 	}
-	
+
 	destination_list = g_list_reverse(destination_list);
 
 	return destination_list;
@@ -456,7 +453,7 @@ void
 e_destination_store_remove_destination_nth (EDestinationStore *destination_store, int n)
 {
 	EDestination *destination;
-	
+
 	g_return_if_fail ( n >= 0);
 
 	destination = g_ptr_array_index(destination_store->destinations, n);
